Package: enmpa
Title: Ecological Niche Modeling using Presence-Absence Data
Version: 0.2.4
Authors@R: c(
    person("Luis F.", "Arias-Giraldo", email = "lfarias.giraldo@gmail.com",
    role = c("aut", "cre"), comment = c(ORCID = "0000-0003-4861-8064")),
    person("Marlon E.", "Cobos", email = "manubio13@gmail.com", 
    role = "aut", comment = c(ORCID = "0000-0002-2611-1767")),
    person("A. Townsend", "Peterson", email = "town@ku.edu", role = "ctb",
    comment = c(ORCID = "0000-0003-0243-2379"))
    )
Maintainer: Luis F. Arias-Giraldo <lfarias.giraldo@gmail.com>
Date: 2026-05-28
Description: A set of tools to perform Ecological Niche Modeling with
    presence-absence data. It includes algorithms for data partitioning, 
    model fitting, calibration, evaluation, selection, and prediction. 
    Other functions help to explore signals of ecological niche using univariate
    and multivariate analyses, and model features such as variable response 
    curves and variable importance. Unique characteristics of this package are
    the ability to exclude models with concave quadratic responses, and the
    option to clamp model predictions to specific variables. These tools are
    implemented following principles proposed in
    Cobos et al., (2022) <doi:10.17161/bi.v17i.15985>, 
    Cobos et al., (2019) <doi:10.7717/peerj.6281>, 
    and Peterson et al., (2008) <doi:10.1016/j.ecolmodel.2007.11.008>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 8.0.0
URL: https://github.com/Luisagi/enmpa
BugReports: https://github.com/Luisagi/enmpa/issues
Imports: doSNOW, ellipse, foreach, graphics, methods, mgcv, parallel,
        Rcpp, snow, stats, terra, utils, vegan
LazyData: true
Depends: R (>= 3.5.0)
LinkingTo: Rcpp
NeedsCompilation: yes
Packaged: 2026-05-28 07:59:54 UTC; luisagi
Author: Luis F. Arias-Giraldo [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4861-8064>),
  Marlon E. Cobos [aut] (ORCID: <https://orcid.org/0000-0002-2611-1767>),
  A. Townsend Peterson [ctb] (ORCID:
    <https://orcid.org/0000-0003-0243-2379>)
Repository: CRAN
Date/Publication: 2026-05-28 08:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-28 11:55:24 UTC; unix
Archs: enmpa.so.dSYM
