# Generated by roxygen2: do not edit by hand

S3method(print,enmpa_calibration)
S3method(print,enmpa_fitted_models)
S3method(summary,enmpa_calibration)
S3method(summary,enmpa_fitted_models)
export(aux_string_comb)
export(aux_var_comb)
export(calibration_glm)
export(evaluation_stats)
export(fit_glms)
export(fit_selected)
export(get_formulas)
export(get_formulas_main)
export(independent_eval01)
export(independent_eval1)
export(jackknife)
export(kfold_partition)
export(model_selection)
export(model_validation)
export(new_enmpa_calibration)
export(new_enmpa_fitted_models)
export(niche_signal)
export(niche_signal_permanova)
export(niche_signal_univariate)
export(optimize_metrics)
export(plot_importance)
export(plot_jk)
export(plot_niche_signal)
export(plot_niche_signal_permanova)
export(plot_niche_signal_univariate)
export(predict_glm)
export(predict_selected)
export(proc_enm)
export(resp2var)
export(response_curve)
export(var_importance)
importFrom(Rcpp,sourceCpp)
importFrom(doSNOW,registerDoSNOW)
importFrom(ellipse,ellipse)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(grDevices,hcl.colors)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(mgcv,gam)
importFrom(parallel,clusterExport)
importFrom(snow,makeSOCKcluster)
importFrom(snow,stopCluster)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cov)
importFrom(stats,deviance)
importFrom(stats,ecdf)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,predict.glm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,update)
importFrom(stats,var)
importFrom(terra,as.data.frame)
importFrom(terra,extract)
importFrom(terra,minmax)
importFrom(terra,predict)
importFrom(terra,rast)
importFrom(terra,setValues)
importFrom(utils,combn)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
importFrom(vegan,adonis2)
useDynLib(enmpa, .registration = TRUE)
